"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ivi_1 = require("ivi");
/**
 * {@link AttributeDirective} with `""` value and {@link updateValue} sync function.
 */
const VALUE_EMPTY = { v: "", u: updateValue };
/**
 * {@link AttributeDirective} with `false` value and {@link updateChecked} sync function.
 */
const CHECKED_FALSE = { v: false, u: updateChecked };
/**
 * {@link AttributeDirective} with `true` value and {@link updateChecked} sync function.
 */
const CHECKED_TRUE = { v: true, u: updateChecked };
/**
 * Synchronization function for {@link AttributeDirective} created with {@link VALUE} function.
 *
 * @param element Target element.
 * @param key Attribute key.
 * @param prev Previous value.
 * @param next Next value.
 */
function updateValue(element, key, prev, next) {
    if (prev === void 0) {
        if (next !== "") {
            element.value = next;
        }
    }
    else if (next !== void 0 && (element.value !== next)) {
        element.value = next;
    }
}
/**
 * Synchronization function for {@link AttributeDirective} created with {@link CHECKED} function.
 *
 * @param element Target element.
 * @param key Attribute key.
 * @param prev Previous value.
 * @param next Next value.
 */
function updateChecked(element, key, prev, next) {
    if (prev === void 0) {
        if (next) {
            element.checked = next;
        }
    }
    else if (next !== void 0) {
        if (element.checked !== next) {
            element.checked = next;
        }
    }
}
/**
 * VALUE function creates a {@link AttributeDirective} that assigns a `value` property to an HTMLInputElement.
 *
 * @example
 *
 *   const e = input("", { value: VALUE("value") });
 *
 * @param v Value.
 * @returns {@link AttributeDirective}
 */
exports.VALUE = (v) => (v === "" ? VALUE_EMPTY : { v, u: updateValue });
/**
 * CONTENT function creates a {@link AttributeDirective} that assigns a `value` property to an HTMLTextAreaElement.
 *
 * @example
 *
 *   const e = textarea("", { content: CONTENT("content") });
 *
 * @param v Value.
 * @returns {@link AttributeDirective}
 */
exports.CONTENT = (exports.VALUE);
/**
 * CHECKED function creates a {@link AttributeDirective} that assigns a `checked` property to an HTMLInputElement.
 *
 * @example
 *
 *   const e = input("", { checked: CHECKED(true) });
 *
 * @param v Checked value.
 * @returns {@link AttributeDirective}
 */
exports.CHECKED = (v) => (v ? CHECKED_TRUE : CHECKED_FALSE);
/* tslint:disable:max-line-length */
/**
 * Creates OpNode HTML element <a>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <a>
 */
exports.a = ivi_1.htmlElementFactory("a");
/**
 * Creates OpNode HTML element <abbr>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/abbr}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <abbr>
 */
exports.abbr = ivi_1.htmlElementFactory("abbr");
/**
 * Creates OpNode HTML element <address>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/address}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <address>
 */
exports.address = ivi_1.htmlElementFactory("address");
/**
 * Creates OpNode HTML element <article>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/article}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <article>
 */
exports.article = ivi_1.htmlElementFactory("article");
/**
 * Creates OpNode HTML element <aside>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/aside}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <aside>
 */
exports.aside = ivi_1.htmlElementFactory("aside");
/**
 * Creates OpNode HTML element <b>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/b}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <b>
 */
exports.b = ivi_1.htmlElementFactory("b");
/**
 * Creates OpNode HTML element <base>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/base}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <base>
 */
exports.base = ivi_1.htmlElementFactory("base");
/**
 * Creates OpNode HTML element <bdo>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/bdo}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <bdo>
 */
exports.bdo = ivi_1.htmlElementFactory("bdo");
/**
 * Creates OpNode HTML element <blockquote>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/blockquote}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <blockquote>
 */
exports.blockquote = ivi_1.htmlElementFactory("blockquote");
/**
 * Creates OpNode HTML element <body>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <body>
 */
exports.body = ivi_1.htmlElementFactory("body");
/**
 * Creates OpNode HTML element <br>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/br}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <br>
 */
exports.br = ivi_1.htmlElementFactory("br");
/**
 * Creates OpNode HTML element <button>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/button}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <button>
 */
exports.button = ivi_1.htmlElementFactory("button");
/**
 * Creates OpNode HTML element <canvas>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/canvas}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <canvas>
 */
exports.canvas = ivi_1.htmlElementFactory("canvas");
/**
 * Creates OpNode HTML element <caption>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/caption}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <caption>
 */
exports.caption = ivi_1.htmlElementFactory("caption");
/**
 * Creates OpNode HTML element <cite>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/cite}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <cite>
 */
exports.cite = ivi_1.htmlElementFactory("cite");
/**
 * Creates OpNode HTML element <code>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/code}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <code>
 */
exports.code = ivi_1.htmlElementFactory("code");
/**
 * Creates OpNode HTML element <col>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/col}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <col>
 */
exports.col = ivi_1.htmlElementFactory("col");
/**
 * Creates OpNode HTML element <colgroup>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/colgroup}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <colgroup>
 */
exports.colgroup = ivi_1.htmlElementFactory("colgroup");
/**
 * Creates OpNode HTML element <del>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/del}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <del>
 */
exports.del = ivi_1.htmlElementFactory("del");
/**
 * Creates OpNode HTML element <dfn>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/dfn}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <dfn>
 */
exports.dfn = ivi_1.htmlElementFactory("dfn");
/**
 * Creates OpNode HTML element <div>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/div}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <div>
 */
exports.div = ivi_1.htmlElementFactory("div");
/**
 * Creates OpNode HTML element <dd>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/dd}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <dd>
 */
exports.dd = ivi_1.htmlElementFactory("dd");
/**
 * Creates OpNode HTML element <dl>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/dl}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <dl>
 */
exports.dl = ivi_1.htmlElementFactory("dl");
/**
 * Creates OpNode HTML element <dt>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/dt}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <dt>
 */
exports.dt = ivi_1.htmlElementFactory("dt");
/**
 * Creates OpNode HTML element <em>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/em}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <em>
 */
exports.em = ivi_1.htmlElementFactory("em");
/**
 * Creates OpNode HTML element <fieldset>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/fieldset}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <fieldset>
 */
exports.fieldset = ivi_1.htmlElementFactory("fieldset");
/**
 * Creates OpNode HTML element <figcaption>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/figcaption}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <figcaption>
 */
exports.figcaption = ivi_1.htmlElementFactory("figcaption");
/**
 * Creates OpNode HTML element <figure>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/figure}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <figure>
 */
exports.figure = ivi_1.htmlElementFactory("figure");
/**
 * Creates OpNode HTML element <footer>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/footer}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <footer>
 */
exports.footer = ivi_1.htmlElementFactory("footer");
/**
 * Creates OpNode HTML element <form>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/form}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <form>
 */
exports.form = ivi_1.htmlElementFactory("form");
/**
 * Creates OpNode HTML element <h1>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/h1}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <h1>
 */
exports.h1 = ivi_1.htmlElementFactory("h1");
/**
 * Creates OpNode HTML element <h2>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/h2}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <h2>
 */
exports.h2 = ivi_1.htmlElementFactory("h2");
/**
 * Creates OpNode HTML element <h3>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/h3}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <h3>
 */
exports.h3 = ivi_1.htmlElementFactory("h3");
/**
 * Creates OpNode HTML element <h4>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/h4}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <h4>
 */
exports.h4 = ivi_1.htmlElementFactory("h4");
/**
 * Creates OpNode HTML element <h5>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/h5}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <h5>
 */
exports.h5 = ivi_1.htmlElementFactory("h5");
/**
 * Creates OpNode HTML element <h6>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/h6}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <h6>
 */
exports.h6 = ivi_1.htmlElementFactory("h6");
/**
 * Creates OpNode HTML element <head>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/head}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <head>
 */
exports.head = ivi_1.htmlElementFactory("head");
/**
 * Creates OpNode HTML element <header>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/header}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <header>
 */
exports.header = ivi_1.htmlElementFactory("header");
/**
 * Creates OpNode HTML element <hgroup>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/hgroup}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <hgroup>
 */
exports.hgroup = ivi_1.htmlElementFactory("hgroup");
/**
 * Creates OpNode HTML element <hr>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/hr}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <hr>
 */
exports.hr = ivi_1.htmlElementFactory("hr");
/**
 * Creates OpNode HTML element <html>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/html}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <html>
 */
exports.html = ivi_1.htmlElementFactory("html");
/**
 * Creates OpNode HTML element <i>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/i}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <i>
 */
exports.i = ivi_1.htmlElementFactory("i");
/**
 * Creates OpNode HTML element <iframe>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/iframe}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <iframe>
 */
exports.iframe = ivi_1.htmlElementFactory("iframe");
/**
 * Creates OpNode HTML element <img>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/img}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <img>
 */
exports.img = ivi_1.htmlElementFactory("img");
/**
 * Creates OpNode HTML element <area>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/area}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <area>
 */
exports.area = ivi_1.htmlElementFactory("area");
/**
 * Creates OpNode HTML element <map>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/map}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <map>
 */
exports.map = ivi_1.htmlElementFactory("map");
/**
 * Creates OpNode HTML element <ins>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/ins}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <ins>
 */
exports.ins = ivi_1.htmlElementFactory("ins");
/**
 * Creates OpNode HTML element <kbd>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/kbd}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <kbd>
 */
exports.kbd = ivi_1.htmlElementFactory("kbd");
/**
 * Creates OpNode HTML element <label>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/label}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <label>
 */
exports.label = ivi_1.htmlElementFactory("label");
/**
 * Creates OpNode HTML element <legend>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/legend}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <legend>
 */
exports.legend = ivi_1.htmlElementFactory("legend");
/**
 * Creates OpNode HTML element <li>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/li}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <li>
 */
exports.li = ivi_1.htmlElementFactory("li");
/**
 * Creates OpNode HTML element <link>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/link}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <link>
 */
exports.link = ivi_1.htmlElementFactory("link");
/**
 * Creates OpNode HTML element <main>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/main}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <main>
 */
exports.main = ivi_1.htmlElementFactory("main");
/**
 * Creates OpNode HTML element <mark>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/mark}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <mark>
 */
exports.mark = ivi_1.htmlElementFactory("mark");
/**
 * Creates OpNode HTML element <menu>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/menu}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <menu>
 */
exports.menu = ivi_1.htmlElementFactory("menu");
/**
 * Creates OpNode HTML element <meta>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/meta}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <meta>
 */
exports.meta = ivi_1.htmlElementFactory("meta");
/**
 * Creates OpNode HTML element <meter>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/meter}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <meter>
 */
exports.meter = ivi_1.htmlElementFactory("meter");
/**
 * Creates OpNode HTML element <nav>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/nav}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <nav>
 */
exports.nav = ivi_1.htmlElementFactory("nav");
/**
 * Creates OpNode HTML element <noscript>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/noscript}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <noscript>
 */
exports.noscript = ivi_1.htmlElementFactory("noscript");
/**
 * Creates OpNode HTML element <ol>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/ol}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <ol>
 */
exports.ol = ivi_1.htmlElementFactory("ol");
/**
 * Creates OpNode HTML element <optgroup>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/optgroup}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <optgroup>
 */
exports.optgroup = ivi_1.htmlElementFactory("optgroup");
/**
 * Creates OpNode HTML element <option>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/option}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <option>
 */
exports.option = ivi_1.htmlElementFactory("option");
/**
 * Creates OpNode HTML element <p>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/p}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <p>
 */
exports.p = ivi_1.htmlElementFactory("p");
/**
 * Creates OpNode HTML element <picture>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/picture}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <picture>
 */
exports.picture = ivi_1.htmlElementFactory("picture");
/**
 * Creates OpNode HTML element <pre>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/pre}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <pre>
 */
exports.pre = ivi_1.htmlElementFactory("pre");
/**
 * Creates OpNode HTML element <progress>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/progress}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <progress>
 */
exports.progress = ivi_1.htmlElementFactory("progress");
/**
 * Creates OpNode HTML element <q>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/q}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <q>
 */
exports.q = ivi_1.htmlElementFactory("q");
/**
 * Creates OpNode HTML element <rt>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/rt}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <rt>
 */
exports.rt = ivi_1.htmlElementFactory("rt");
/**
 * Creates OpNode HTML element <ruby>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/ruby}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <ruby>
 */
exports.ruby = ivi_1.htmlElementFactory("ruby");
/**
 * Creates OpNode HTML element <s>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/s}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <s>
 */
exports.s = ivi_1.htmlElementFactory("s");
/**
 * Creates OpNode HTML element <samp>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/samp}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <samp>
 */
exports.samp = ivi_1.htmlElementFactory("samp");
/**
 * Creates OpNode HTML element <script>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/script}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <script>
 */
exports.script = ivi_1.htmlElementFactory("script");
/**
 * Creates OpNode HTML element <section>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/section}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <section>
 */
exports.section = ivi_1.htmlElementFactory("section");
/**
 * Creates OpNode HTML element <select>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/select}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <select>
 */
exports.select = ivi_1.htmlElementFactory("select");
/**
 * Creates OpNode HTML element <source>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/source}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <source>
 */
exports.source = ivi_1.htmlElementFactory("source");
/**
 * Creates OpNode HTML element <span>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/span}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <span>
 */
exports.span = ivi_1.htmlElementFactory("span");
/**
 * Creates OpNode HTML element <strong>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/strong}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <strong>
 */
exports.strong = ivi_1.htmlElementFactory("strong");
/**
 * Creates OpNode HTML element <style>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/style}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <style>
 */
exports.style = ivi_1.htmlElementFactory("style");
/**
 * Creates OpNode HTML element <sub>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/sub}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <sub>
 */
exports.sub = ivi_1.htmlElementFactory("sub");
/**
 * Creates OpNode HTML element <sup>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/sup}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <sup>
 */
exports.sup = ivi_1.htmlElementFactory("sup");
/**
 * Creates OpNode HTML element <table>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/table}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <table>
 */
exports.table = ivi_1.htmlElementFactory("table");
/**
 * Creates OpNode HTML element <tbody>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/tbody}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <tbody>
 */
exports.tbody = ivi_1.htmlElementFactory("tbody");
/**
 * Creates OpNode HTML element <td>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/td}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <td>
 */
exports.td = ivi_1.htmlElementFactory("td");
/**
 * Creates OpNode HTML element <template>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/template}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <template>
 */
exports.template = ivi_1.htmlElementFactory("template");
/**
 * Creates OpNode HTML element <tfoot>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/tfoot}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <tfoot>
 */
exports.tfoot = ivi_1.htmlElementFactory("tfoot");
/**
 * Creates OpNode HTML element <th>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/th}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <th>
 */
exports.th = ivi_1.htmlElementFactory("th");
/**
 * Creates OpNode HTML element <thead>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/thead}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <thead>
 */
exports.thead = ivi_1.htmlElementFactory("thead");
/**
 * Creates OpNode HTML element <time>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/time}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <time>
 */
exports.time = ivi_1.htmlElementFactory("time");
/**
 * Creates OpNode HTML element <title>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/title}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <title>
 */
exports.title = ivi_1.htmlElementFactory("title");
/**
 * Creates OpNode HTML element <tr>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/tr}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <tr>
 */
exports.tr = ivi_1.htmlElementFactory("tr");
/**
 * Creates OpNode HTML element <track>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/track}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <track>
 */
exports.track = ivi_1.htmlElementFactory("track");
/**
 * Creates OpNode HTML element <u>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/u}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <u>
 */
exports.u = ivi_1.htmlElementFactory("u");
/**
 * Creates OpNode HTML element <ul>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/ul}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <ul>
 */
exports.ul = ivi_1.htmlElementFactory("ul");
/**
 * Creates OpNode HTML element <wbr>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/wbr}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <wbr>
 */
exports.wbr = ivi_1.htmlElementFactory("wbr");
/**
 * Creates OpNode HTML element <textarea>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/textarea}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <textarea>
 */
exports.textarea = ivi_1.htmlElementFactory("textarea");
/**
 * Creates OpNode HTML element <input>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <input>
 */
exports.input = ivi_1.htmlElementFactory("input");
/**
 * Creates OpNode HTML element <audio>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/audio}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <audio>
 */
exports.audio = ivi_1.htmlElementFactory("audio");
/**
 * Creates OpNode HTML element <video>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/video}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <video>
 */
exports.video = ivi_1.htmlElementFactory("video");
//# sourceMappingURL=index.js.map