import { htmlElementFactory, } from "ivi";
/**
 * {@link AttributeDirective} with `""` value and {@link updateValue} sync function.
 */
const VALUE_EMPTY = { v: "", u: updateValue };
/**
 * {@link AttributeDirective} with `false` value and {@link updateChecked} sync function.
 */
const CHECKED_FALSE = { v: false, u: updateChecked };
/**
 * {@link AttributeDirective} with `true` value and {@link updateChecked} sync function.
 */
const CHECKED_TRUE = { v: true, u: updateChecked };
/**
 * Synchronization function for {@link AttributeDirective} created with {@link VALUE} function.
 *
 * @param element Target element.
 * @param key Attribute key.
 * @param prev Previous value.
 * @param next Next value.
 */
function updateValue(element, key, prev, next) {
    if (prev === void 0) {
        if (next !== "") {
            element.value = next;
        }
    }
    else if (next !== void 0 && (element.value !== next)) {
        element.value = next;
    }
}
/**
 * Synchronization function for {@link AttributeDirective} created with {@link CHECKED} function.
 *
 * @param element Target element.
 * @param key Attribute key.
 * @param prev Previous value.
 * @param next Next value.
 */
function updateChecked(element, key, prev, next) {
    if (prev === void 0) {
        if (next) {
            element.checked = next;
        }
    }
    else if (next !== void 0) {
        if (element.checked !== next) {
            element.checked = next;
        }
    }
}
/**
 * VALUE function creates a {@link AttributeDirective} that assigns a `value` property to an HTMLInputElement.
 *
 * @example
 *
 *   const e = input("", { value: VALUE("value") });
 *
 * @param v Value.
 * @returns {@link AttributeDirective}
 */
export const VALUE = (v) => (v === "" ? VALUE_EMPTY : { v, u: updateValue });
/**
 * CONTENT function creates a {@link AttributeDirective} that assigns a `value` property to an HTMLTextAreaElement.
 *
 * @example
 *
 *   const e = textarea("", { content: CONTENT("content") });
 *
 * @param v Value.
 * @returns {@link AttributeDirective}
 */
export const CONTENT = (VALUE);
/**
 * CHECKED function creates a {@link AttributeDirective} that assigns a `checked` property to an HTMLInputElement.
 *
 * @example
 *
 *   const e = input("", { checked: CHECKED(true) });
 *
 * @param v Checked value.
 * @returns {@link AttributeDirective}
 */
export const CHECKED = (v) => (v ? CHECKED_TRUE : CHECKED_FALSE);
/* tslint:disable:max-line-length */
/**
 * Creates OpNode HTML element <a>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <a>
 */
export const a = /*#__PURE__*/ htmlElementFactory("a");
/**
 * Creates OpNode HTML element <abbr>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/abbr}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <abbr>
 */
export const abbr = /*#__PURE__*/ htmlElementFactory("abbr");
/**
 * Creates OpNode HTML element <address>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/address}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <address>
 */
export const address = /*#__PURE__*/ htmlElementFactory("address");
/**
 * Creates OpNode HTML element <article>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/article}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <article>
 */
export const article = /*#__PURE__*/ htmlElementFactory("article");
/**
 * Creates OpNode HTML element <aside>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/aside}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <aside>
 */
export const aside = /*#__PURE__*/ htmlElementFactory("aside");
/**
 * Creates OpNode HTML element <b>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/b}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <b>
 */
export const b = /*#__PURE__*/ htmlElementFactory("b");
/**
 * Creates OpNode HTML element <base>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/base}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <base>
 */
export const base = /*#__PURE__*/ htmlElementFactory("base");
/**
 * Creates OpNode HTML element <bdo>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/bdo}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <bdo>
 */
export const bdo = /*#__PURE__*/ htmlElementFactory("bdo");
/**
 * Creates OpNode HTML element <blockquote>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/blockquote}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <blockquote>
 */
export const blockquote = /*#__PURE__*/ htmlElementFactory("blockquote");
/**
 * Creates OpNode HTML element <body>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <body>
 */
export const body = /*#__PURE__*/ htmlElementFactory("body");
/**
 * Creates OpNode HTML element <br>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/br}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <br>
 */
export const br = /*#__PURE__*/ htmlElementFactory("br");
/**
 * Creates OpNode HTML element <button>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/button}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <button>
 */
export const button = /*#__PURE__*/ htmlElementFactory("button");
/**
 * Creates OpNode HTML element <canvas>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/canvas}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <canvas>
 */
export const canvas = /*#__PURE__*/ htmlElementFactory("canvas");
/**
 * Creates OpNode HTML element <caption>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/caption}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <caption>
 */
export const caption = /*#__PURE__*/ htmlElementFactory("caption");
/**
 * Creates OpNode HTML element <cite>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/cite}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <cite>
 */
export const cite = /*#__PURE__*/ htmlElementFactory("cite");
/**
 * Creates OpNode HTML element <code>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/code}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <code>
 */
export const code = /*#__PURE__*/ htmlElementFactory("code");
/**
 * Creates OpNode HTML element <col>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/col}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <col>
 */
export const col = /*#__PURE__*/ htmlElementFactory("col");
/**
 * Creates OpNode HTML element <colgroup>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/colgroup}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <colgroup>
 */
export const colgroup = /*#__PURE__*/ htmlElementFactory("colgroup");
/**
 * Creates OpNode HTML element <del>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/del}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <del>
 */
export const del = /*#__PURE__*/ htmlElementFactory("del");
/**
 * Creates OpNode HTML element <dfn>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/dfn}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <dfn>
 */
export const dfn = /*#__PURE__*/ htmlElementFactory("dfn");
/**
 * Creates OpNode HTML element <div>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/div}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <div>
 */
export const div = /*#__PURE__*/ htmlElementFactory("div");
/**
 * Creates OpNode HTML element <dd>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/dd}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <dd>
 */
export const dd = /*#__PURE__*/ htmlElementFactory("dd");
/**
 * Creates OpNode HTML element <dl>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/dl}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <dl>
 */
export const dl = /*#__PURE__*/ htmlElementFactory("dl");
/**
 * Creates OpNode HTML element <dt>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/dt}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <dt>
 */
export const dt = /*#__PURE__*/ htmlElementFactory("dt");
/**
 * Creates OpNode HTML element <em>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/em}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <em>
 */
export const em = /*#__PURE__*/ htmlElementFactory("em");
/**
 * Creates OpNode HTML element <fieldset>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/fieldset}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <fieldset>
 */
export const fieldset = /*#__PURE__*/ htmlElementFactory("fieldset");
/**
 * Creates OpNode HTML element <figcaption>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/figcaption}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <figcaption>
 */
export const figcaption = /*#__PURE__*/ htmlElementFactory("figcaption");
/**
 * Creates OpNode HTML element <figure>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/figure}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <figure>
 */
export const figure = /*#__PURE__*/ htmlElementFactory("figure");
/**
 * Creates OpNode HTML element <footer>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/footer}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <footer>
 */
export const footer = /*#__PURE__*/ htmlElementFactory("footer");
/**
 * Creates OpNode HTML element <form>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/form}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <form>
 */
export const form = /*#__PURE__*/ htmlElementFactory("form");
/**
 * Creates OpNode HTML element <h1>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/h1}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <h1>
 */
export const h1 = /*#__PURE__*/ htmlElementFactory("h1");
/**
 * Creates OpNode HTML element <h2>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/h2}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <h2>
 */
export const h2 = /*#__PURE__*/ htmlElementFactory("h2");
/**
 * Creates OpNode HTML element <h3>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/h3}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <h3>
 */
export const h3 = /*#__PURE__*/ htmlElementFactory("h3");
/**
 * Creates OpNode HTML element <h4>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/h4}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <h4>
 */
export const h4 = /*#__PURE__*/ htmlElementFactory("h4");
/**
 * Creates OpNode HTML element <h5>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/h5}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <h5>
 */
export const h5 = /*#__PURE__*/ htmlElementFactory("h5");
/**
 * Creates OpNode HTML element <h6>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/h6}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <h6>
 */
export const h6 = /*#__PURE__*/ htmlElementFactory("h6");
/**
 * Creates OpNode HTML element <head>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/head}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <head>
 */
export const head = /*#__PURE__*/ htmlElementFactory("head");
/**
 * Creates OpNode HTML element <header>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/header}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <header>
 */
export const header = /*#__PURE__*/ htmlElementFactory("header");
/**
 * Creates OpNode HTML element <hgroup>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/hgroup}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <hgroup>
 */
export const hgroup = /*#__PURE__*/ htmlElementFactory("hgroup");
/**
 * Creates OpNode HTML element <hr>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/hr}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <hr>
 */
export const hr = /*#__PURE__*/ htmlElementFactory("hr");
/**
 * Creates OpNode HTML element <html>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/html}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <html>
 */
export const html = /*#__PURE__*/ htmlElementFactory("html");
/**
 * Creates OpNode HTML element <i>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/i}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <i>
 */
export const i = /*#__PURE__*/ htmlElementFactory("i");
/**
 * Creates OpNode HTML element <iframe>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/iframe}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <iframe>
 */
export const iframe = /*#__PURE__*/ htmlElementFactory("iframe");
/**
 * Creates OpNode HTML element <img>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/img}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <img>
 */
export const img = /*#__PURE__*/ htmlElementFactory("img");
/**
 * Creates OpNode HTML element <area>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/area}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <area>
 */
export const area = /*#__PURE__*/ htmlElementFactory("area");
/**
 * Creates OpNode HTML element <map>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/map}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <map>
 */
export const map = /*#__PURE__*/ htmlElementFactory("map");
/**
 * Creates OpNode HTML element <ins>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/ins}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <ins>
 */
export const ins = /*#__PURE__*/ htmlElementFactory("ins");
/**
 * Creates OpNode HTML element <kbd>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/kbd}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <kbd>
 */
export const kbd = /*#__PURE__*/ htmlElementFactory("kbd");
/**
 * Creates OpNode HTML element <label>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/label}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <label>
 */
export const label = /*#__PURE__*/ htmlElementFactory("label");
/**
 * Creates OpNode HTML element <legend>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/legend}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <legend>
 */
export const legend = /*#__PURE__*/ htmlElementFactory("legend");
/**
 * Creates OpNode HTML element <li>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/li}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <li>
 */
export const li = /*#__PURE__*/ htmlElementFactory("li");
/**
 * Creates OpNode HTML element <link>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/link}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <link>
 */
export const link = /*#__PURE__*/ htmlElementFactory("link");
/**
 * Creates OpNode HTML element <main>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/main}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <main>
 */
export const main = /*#__PURE__*/ htmlElementFactory("main");
/**
 * Creates OpNode HTML element <mark>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/mark}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <mark>
 */
export const mark = /*#__PURE__*/ htmlElementFactory("mark");
/**
 * Creates OpNode HTML element <menu>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/menu}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <menu>
 */
export const menu = /*#__PURE__*/ htmlElementFactory("menu");
/**
 * Creates OpNode HTML element <meta>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/meta}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <meta>
 */
export const meta = /*#__PURE__*/ htmlElementFactory("meta");
/**
 * Creates OpNode HTML element <meter>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/meter}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <meter>
 */
export const meter = /*#__PURE__*/ htmlElementFactory("meter");
/**
 * Creates OpNode HTML element <nav>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/nav}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <nav>
 */
export const nav = /*#__PURE__*/ htmlElementFactory("nav");
/**
 * Creates OpNode HTML element <noscript>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/noscript}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <noscript>
 */
export const noscript = /*#__PURE__*/ htmlElementFactory("noscript");
/**
 * Creates OpNode HTML element <ol>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/ol}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <ol>
 */
export const ol = /*#__PURE__*/ htmlElementFactory("ol");
/**
 * Creates OpNode HTML element <optgroup>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/optgroup}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <optgroup>
 */
export const optgroup = /*#__PURE__*/ htmlElementFactory("optgroup");
/**
 * Creates OpNode HTML element <option>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/option}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <option>
 */
export const option = /*#__PURE__*/ htmlElementFactory("option");
/**
 * Creates OpNode HTML element <p>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/p}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <p>
 */
export const p = /*#__PURE__*/ htmlElementFactory("p");
/**
 * Creates OpNode HTML element <picture>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/picture}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <picture>
 */
export const picture = /*#__PURE__*/ htmlElementFactory("picture");
/**
 * Creates OpNode HTML element <pre>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/pre}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <pre>
 */
export const pre = /*#__PURE__*/ htmlElementFactory("pre");
/**
 * Creates OpNode HTML element <progress>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/progress}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <progress>
 */
export const progress = /*#__PURE__*/ htmlElementFactory("progress");
/**
 * Creates OpNode HTML element <q>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/q}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <q>
 */
export const q = /*#__PURE__*/ htmlElementFactory("q");
/**
 * Creates OpNode HTML element <rt>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/rt}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <rt>
 */
export const rt = /*#__PURE__*/ htmlElementFactory("rt");
/**
 * Creates OpNode HTML element <ruby>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/ruby}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <ruby>
 */
export const ruby = /*#__PURE__*/ htmlElementFactory("ruby");
/**
 * Creates OpNode HTML element <s>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/s}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <s>
 */
export const s = /*#__PURE__*/ htmlElementFactory("s");
/**
 * Creates OpNode HTML element <samp>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/samp}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <samp>
 */
export const samp = /*#__PURE__*/ htmlElementFactory("samp");
/**
 * Creates OpNode HTML element <script>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/script}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <script>
 */
export const script = /*#__PURE__*/ htmlElementFactory("script");
/**
 * Creates OpNode HTML element <section>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/section}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <section>
 */
export const section = /*#__PURE__*/ htmlElementFactory("section");
/**
 * Creates OpNode HTML element <select>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/select}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <select>
 */
export const select = /*#__PURE__*/ htmlElementFactory("select");
/**
 * Creates OpNode HTML element <source>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/source}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <source>
 */
export const source = /*#__PURE__*/ htmlElementFactory("source");
/**
 * Creates OpNode HTML element <span>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/span}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <span>
 */
export const span = /*#__PURE__*/ htmlElementFactory("span");
/**
 * Creates OpNode HTML element <strong>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/strong}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <strong>
 */
export const strong = /*#__PURE__*/ htmlElementFactory("strong");
/**
 * Creates OpNode HTML element <style>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/style}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <style>
 */
export const style = /*#__PURE__*/ htmlElementFactory("style");
/**
 * Creates OpNode HTML element <sub>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/sub}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <sub>
 */
export const sub = /*#__PURE__*/ htmlElementFactory("sub");
/**
 * Creates OpNode HTML element <sup>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/sup}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <sup>
 */
export const sup = /*#__PURE__*/ htmlElementFactory("sup");
/**
 * Creates OpNode HTML element <table>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/table}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <table>
 */
export const table = /*#__PURE__*/ htmlElementFactory("table");
/**
 * Creates OpNode HTML element <tbody>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/tbody}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <tbody>
 */
export const tbody = /*#__PURE__*/ htmlElementFactory("tbody");
/**
 * Creates OpNode HTML element <td>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/td}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <td>
 */
export const td = /*#__PURE__*/ htmlElementFactory("td");
/**
 * Creates OpNode HTML element <template>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/template}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <template>
 */
export const template = /*#__PURE__*/ htmlElementFactory("template");
/**
 * Creates OpNode HTML element <tfoot>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/tfoot}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <tfoot>
 */
export const tfoot = /*#__PURE__*/ htmlElementFactory("tfoot");
/**
 * Creates OpNode HTML element <th>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/th}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <th>
 */
export const th = /*#__PURE__*/ htmlElementFactory("th");
/**
 * Creates OpNode HTML element <thead>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/thead}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <thead>
 */
export const thead = /*#__PURE__*/ htmlElementFactory("thead");
/**
 * Creates OpNode HTML element <time>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/time}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <time>
 */
export const time = /*#__PURE__*/ htmlElementFactory("time");
/**
 * Creates OpNode HTML element <title>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/title}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <title>
 */
export const title = /*#__PURE__*/ htmlElementFactory("title");
/**
 * Creates OpNode HTML element <tr>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/tr}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <tr>
 */
export const tr = /*#__PURE__*/ htmlElementFactory("tr");
/**
 * Creates OpNode HTML element <track>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/track}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <track>
 */
export const track = /*#__PURE__*/ htmlElementFactory("track");
/**
 * Creates OpNode HTML element <u>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/u}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <u>
 */
export const u = /*#__PURE__*/ htmlElementFactory("u");
/**
 * Creates OpNode HTML element <ul>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/ul}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <ul>
 */
export const ul = /*#__PURE__*/ htmlElementFactory("ul");
/**
 * Creates OpNode HTML element <wbr>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/wbr}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <wbr>
 */
export const wbr = /*#__PURE__*/ htmlElementFactory("wbr");
/**
 * Creates OpNode HTML element <textarea>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/textarea}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <textarea>
 */
export const textarea = /*#__PURE__*/ htmlElementFactory("textarea");
/**
 * Creates OpNode HTML element <input>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <input>
 */
export const input = /*#__PURE__*/ htmlElementFactory("input");
/**
 * Creates OpNode HTML element <audio>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/audio}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <audio>
 */
export const audio = /*#__PURE__*/ htmlElementFactory("audio");
/**
 * Creates OpNode HTML element <video>.
 *
 * {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/video}
 *
 * @param className Class name.
 * @param attrs Attributes.
 * @param children Children nodes.
 * @returns OpNode HTML element <video>
 */
export const video = /*#__PURE__*/ htmlElementFactory("video");
//# sourceMappingURL=index.js.map